@@ -4,14 +4,29 @@
 
 #include <NetworkManagerQt/GsmSetting>
 #include <NetworkManagerQt/Manager>
+#include <NetworkManagerQt/Settings>
+#include <NetworkManagerQt/Utils>
 
 #include "signalindicator.h"
 
-SignalIndicator::SignalIndicator()
+SignalIndicator::SignalIndicator(QObject *parent)
+    : QObject{parent}
+    , m_nmModem{nullptr}
+    , m_modemDevice{nullptr}
+    , m_modem{nullptr}
+    , m_3gppModem{nullptr}
 {
-    connect(ModemManager::notifier(), &ModemManager::Notifier::modemAdded, this, &SignalIndicator::updateModem);
-    connect(ModemManager::notifier(), &ModemManager::Notifier::modemRemoved, this, &SignalIndicator::updateModem);
-    updateModem();
+    connect(ModemManager::notifier(), &ModemManager::Notifier::modemAdded, this, &SignalIndicator::updateModemManagerModem);
+    connect(ModemManager::notifier(), &ModemManager::Notifier::modemRemoved, this, &SignalIndicator::updateModemManagerModem);
+
+    connect(NetworkManager::settingsNotifier(), &NetworkManager::SettingsNotifier::connectionAdded, this, &SignalIndicator::mobileDataEnabledChanged);
+    connect(NetworkManager::settingsNotifier(), &NetworkManager::SettingsNotifier::connectionRemoved, this, &SignalIndicator::mobileDataEnabledChanged);
+    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionAdded, this, &SignalIndicator::mobileDataEnabledChanged);
+    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionRemoved, this, &SignalIndicator::mobileDataEnabledChanged);
+    connect(NetworkManager::notifier(), &NetworkManager::Notifier::deviceAdded, this, &SignalIndicator::updateNetworkManagerModem);
+    connect(NetworkManager::notifier(), &NetworkManager::Notifier::deviceRemoved, this, &SignalIndicator::updateNetworkManagerModem);
+
+    updateModemManagerModem();
 }
 
 int SignalIndicator::strength() const
@@ -27,6 +42,11 @@ QString SignalIndicator::name() const
     return m_3gppModem ? m_3gppModem->operatorName() : QString();
 }
 
+bool SignalIndicator::modemAvailable() const
+{
+    return !m_modem.isNull();
+}
+
 bool SignalIndicator::simLocked() const
 {
     if (!m_modem) {
@@ -37,20 +57,12 @@ bool SignalIndicator::simLocked() const
 
 bool SignalIndicator::simEmpty() const
 {
-    if (!m_modem) {
-        return false;
-    }
-    return m_modemDevice && m_modemDevice->sim() && m_modemDevice->sim()->uni() == QStringLiteral("/");
-}
-
-bool SignalIndicator::available() const
-{
-    return !ModemManager::modemDevices().isEmpty();
+    return !m_modemDevice || !m_modemDevice->sim() || (m_modemDevice->sim()->uni() == QStringLiteral("/"));
 }
 
 bool SignalIndicator::mobileDataSupported() const
 {
-    return m_nmModem && m_modemDevice->sim() && !simEmpty();
+    return m_nmModem && !simEmpty();
 }
 
 bool SignalIndicator::mobileDataEnabled() const
@@ -98,7 +110,7 @@ void SignalIndicator::setMobileDataEnabled(bool enabled)
             con->settings()->setAutoconnect(false);
             con->update(con->settings()->toMap());
         }
-        m_nmModem->disconnectInterface().waitForFinished();
+        m_nmModem->disconnectInterface();
     } else {
         m_nmModem->setAutoconnect(true);
         // activate the connection that was last used
@@ -125,32 +137,22 @@ void SignalIndicator::setMobileDataEnabled(bool enabled)
     }
 }
 
-void SignalIndicator::updateModem()
+void SignalIndicator::updateModemManagerModem()
 {
-    if (!available()) {
+    m_modemDevice = nullptr;
+    m_modem = nullptr;
+    m_3gppModem = nullptr;
+
+    if (ModemManager::modemDevices().isEmpty()) {
         qWarning() << "No modems available";
         return;
     }
 
-    // we assume that there is a single modem
+    // TODO: we assume that there is a single modem for the time being
     m_modemDevice = ModemManager::modemDevices()[0];
     m_modem = m_modemDevice->modemInterface();
     m_3gppModem = m_modemDevice->interface(ModemManager::ModemDevice::GsmInterface).objectCast<ModemManager::Modem3gpp>();
 
-    // find networkmanager modem
-    for (NetworkManager::Device::Ptr nmDevice : NetworkManager::networkInterfaces()) {
-        if (nmDevice->udi() == m_modemDevice->uni()) {
-            m_nmModem = nmDevice.objectCast<NetworkManager::ModemDevice>();
-
-            connect(m_nmModem.get(), &NetworkManager::Device::autoconnectChanged, this, [this]() {
-                Q_EMIT mobileDataEnabledChanged();
-            });
-            connect(m_nmModem.get(), &NetworkManager::Device::stateChanged, this, [this](auto, auto, auto) {
-                Q_EMIT mobileDataEnabledChanged();
-            });
-        }
-    }
-    
     connect(m_modemDevice->sim().get(), &ModemManager::Sim::simIdentifierChanged, this, &SignalIndicator::simEmptyChanged);
 
     if (m_modem) {
@@ -161,7 +163,32 @@ void SignalIndicator::updateModem()
         connect(m_3gppModem.get(), &ModemManager::Modem3gpp::operatorNameChanged, this, &SignalIndicator::nameChanged);
     }
 
-    Q_EMIT mobileDataSupportedChanged();
+    updateNetworkManagerModem();
+
     Q_EMIT nameChanged();
-    Q_EMIT availableChanged();
+    Q_EMIT strengthChanged();
+    Q_EMIT modemAvailableChanged();
+}
+
+void SignalIndicator::updateNetworkManagerModem()
+{
+    m_nmModem = nullptr;
+    if (!m_modemDevice) {
+        return;
+    }
+
+    // find networkmanager modem
+    for (NetworkManager::Device::Ptr nmDevice : NetworkManager::networkInterfaces()) {
+        if (nmDevice->udi() == m_modemDevice->uni()) {
+            m_nmModem = nmDevice.objectCast<NetworkManager::ModemDevice>();
+
+            connect(m_nmModem.get(), &NetworkManager::Device::autoconnectChanged, this, &SignalIndicator::mobileDataEnabledChanged);
+            connect(m_nmModem.get(), &NetworkManager::Device::stateChanged, this, &SignalIndicator::mobileDataEnabledChanged);
+            connect(m_nmModem.get(), &NetworkManager::Device::availableConnectionAppeared, this, &SignalIndicator::mobileDataEnabledChanged);
+            connect(m_nmModem.get(), &NetworkManager::Device::availableConnectionDisappeared, this, &SignalIndicator::mobileDataEnabledChanged);
+        }
+    }
+
+    Q_EMIT mobileDataSupportedChanged();
+    Q_EMIT mobileDataEnabledChanged();
 }
